@@ -76,7 +76,7 @@ const LMASK: u64 = !CMASK;
 
 // HLC Delta in milliseconds: maximum accepted drift for an external timestamp.
 // I.e.: if an incoming timestamp has a time > now() + delta, then the HLC is not updated.
-const DEFAULT_DELTA_MS: u64 = 100;
+const DEFAULT_DELTA_MS: u64 = 500;
 lazy_static! {
     static ref DELTA_MS: u64 = match var("UHLC_MAX_DELTA_MS") {
         Ok(s) => s.parse().unwrap_or_else(|e| panic!(
@@ -91,6 +91,90 @@ lazy_static! {
     };
 }
 
+///
+/// The builder of [`HLC`].
+///
+/// # Examples
+///
+/// ```
+/// use std::convert::TryFrom;
+/// use std::time::Duration;
+/// use uhlc::{HLCBuilder, ID};
+///
+/// let default_hlc = HLCBuilder::new().build();
+/// println!("{}", default_hlc.new_timestamp());
+///
+/// let custom_hlc = HLCBuilder::new()
+///    .with_id(ID::try_from(vec![0x01, 0x02, 0x03].as_ref()).unwrap())
+///    .with_max_delta(Duration::from_secs(1))
+///    .build();
+/// println!("{}", custom_hlc.new_timestamp());
+pub struct HLCBuilder {
+    hlc: HLC,
+}
+
+impl HLCBuilder {
+    ///
+    /// Constructs a new HLCBuilder for the creation of an [`HLC`], with the following default configuration:
+    ///  * a random UUID as HLC identifier.
+    ///   Can be changed calling [`Self::with_id()`].
+    ///  * [`system_time_clock()`] as physical clock (i.e. the ).
+    ///   Can be changed calling [`Self::with_clock()`].
+    ///  * 500 millisecond as maximum delta (i.e. the maximum accepted drift for an external timestamp).
+    ///   Can be changed calling [`Self::with_max_delta()`].
+    ///
+    pub fn new() -> HLCBuilder {
+        HLCBuilder::default()
+    }
+
+    ///
+    /// Configure a specific identifier for the HLC to be created.
+    ///
+    /// **NOTE: the identifier must be unique in the system.**
+    ///
+    pub fn with_id(mut self, id: ID) -> HLCBuilder {
+        self.hlc.id = id;
+        self
+    }
+
+    ///
+    /// Configure a specific physical clock for the HLC to be created.
+    ///
+    /// The `clock` parameter must be a function returning a new physical time (as an [`NTP64`] at each call.  
+    /// The time returned by this clock doesn't need to be monotonic: when the HLC generates a new timestamp from this time,
+    /// it first checks if this time is greater than the previously generated timestamp. If not, the new timestamp it the previous one +1.
+    ///
+    pub fn with_clock(mut self, clock: fn() -> NTP64) -> HLCBuilder {
+        self.hlc.clock = clock;
+        self
+    }
+
+    ///
+    /// Configure the maximum delta accepted by an HLC when updating it's logical clock calling [`HLC::update_with_timestamp()`].
+    ///
+    pub fn with_max_delta(mut self, delta: Duration) -> HLCBuilder {
+        self.hlc.delta = delta.into();
+        self
+    }
+
+    pub fn build(self) -> HLC {
+        self.hlc
+    }
+}
+
+impl Default for HLCBuilder {
+    fn default() -> Self {
+        HLCBuilder {
+            hlc: HLC {
+                id: uuid::Uuid::new_v4().into(),
+                clock: system_time_clock,
+                delta: NTP64::from(Duration::from_millis(*DELTA_MS)),
+                last_time: Default::default(),
+            },
+        }
+    }
+}
+
 /// An Hybric Logical Clock generating [`Timestamp`]s
 pub struct HLC {
     id: ID,
@@ -109,44 +193,6 @@ macro_rules! lock {
 }
 
 impl HLC {
-    /// Create a new [`HLC`] with an `id` (must be unique) and a `clock`
-    /// (a function returning the current physical time as an [`NTP64`]).
-    ///
-    /// # Examples
-    ///
-    /// ```
-    /// use uhlc::HLC;
-    ///
-    /// let hlc = HLC::with_clock(
-    ///     uuid::Uuid::new_v4().into(),
-    ///     uhlc::system_time_clock);
-    /// println!("{}", hlc.new_timestamp());
-    /// ```
-    pub fn with_clock(id: ID, clock: fn() -> NTP64) -> HLC {
-        let delta = NTP64::from(Duration::from_millis(*DELTA_MS));
-        HLC {
-            id,
-            clock,
-            delta,
-            last_time: Default::default(),
-        }
-    }
-
-    /// Create a new [`HLC`] with an `id` (must be unique) and using
-    /// [`system_time_clock()`] as physical clock.
-    ///
-    /// # Examples
-    ///
-    /// ```
-    /// use uhlc::HLC;
-    ///
-    /// let hlc = HLC::with_system_time(uuid::Uuid::new_v4().into());
-    /// println!("{}", hlc.new_timestamp());
-    /// ```
-    pub fn with_system_time(id: ID) -> HLC {
-        HLC::with_clock(id, system_time_clock)
-    }
-
     /// Generate a new [`Timestamp`].
     ///
     /// This timestamp is unique in the system and is always greater
@@ -179,8 +225,8 @@ impl HLC {
     /// Update this [`HLC`] with a [`Timestamp`].
     ///
     /// Typically, this timestamp should have been generated by another HLC.
-    /// If the timestamp exceeds the current time of this HLC by more than [`DELTA_MS`]
-    /// an [`Err`] is returned.
+    /// If the timestamp exceeds the current time of this HLC by more than the configured maximum delta
+    /// (see [`HLCBuilder::with_max_delta()`]) an [`Err`] is returned.
     ///
     /// # Examples
     ///
@@ -233,21 +279,17 @@ impl HLC {
 impl Default for HLC {
     /// Create a new [`HLC`] with a generated UUID and using
     /// [`system_time_clock()`] as physical clock.
+    /// This is equivalent to `HLCBuilder::default().build()`
     fn default() -> Self {
-        HLC::with_system_time(uuid::Uuid::new_v4().into())
+        HLCBuilder::default().build()
     }
 }
 
 /// A physical clock relying on std::time::SystemTime::now().
 ///
-/// It returns a NTP64 relative to std::time::UNIX_EPOCH (1st Jan 1970).
+/// It returns a NTP64 relative to std::time::UNIX_EPOCH (1st Jan 1970).  
+/// That's the default clock used by an [`HLC`] if [`HLCBuilder::with_clock()`] is not called.
 ///
-/// # Examples
-///
-/// ```
-/// let hlc = uhlc::HLC::with_clock(uuid::Uuid::new_v4().into(), uhlc::system_time_clock);
-/// println!("{}", hlc.new_timestamp());
-/// ```
 #[inline]
 pub fn system_time_clock() -> NTP64 {
     NTP64::from(SystemTime::now().duration_since(UNIX_EPOCH).unwrap())
@@ -281,10 +323,10 @@ mod tests {
             let id1: ID = ID::try_from(vec![0x01].as_ref()).unwrap();
             let id2: ID = ID::try_from(vec![0x02].as_ref()).unwrap();
             let id3: ID = ID::try_from(vec![0x03].as_ref()).unwrap();
-            let hlc0 = Arc::new(HLC::with_system_time(id0.clone()));
-            let hlc1 = Arc::new(HLC::with_system_time(id1.clone()));
-            let hlc2 = Arc::new(HLC::with_system_time(id2.clone()));
-            let hlc3 = Arc::new(HLC::with_system_time(id3.clone()));
+            let hlc0 = Arc::new(HLCBuilder::new().with_id(id0.clone()).build());
+            let hlc1 = Arc::new(HLCBuilder::new().with_id(id1.clone()).build());
+            let hlc2 = Arc::new(HLCBuilder::new().with_id(id2.clone()).build());
+            let hlc3 = Arc::new(HLCBuilder::new().with_id(id3.clone()).build());
 
             // Make 4 tasks to generate 10000 timestamps each with distinct HLCs,
             // and also to update each other HLCs
@@ -367,7 +409,7 @@ mod tests {
     #[test]
     fn hlc_update_with_timestamp() {
         let id: ID = ID::from(uuid::Uuid::new_v4());
-        let hlc = HLC::with_system_time(id.clone());
+        let hlc = HLCBuilder::new().with_id(id.clone()).build();
 
         // Test that updating with an old Timestamp don't break the HLC
         let past_ts = Timestamp::new(Default::default(), id.clone());
@@ -377,7 +419,7 @@ mod tests {
 
         // Test that updating with a Timestamp exceeding the delta is refused
         let now_ts = hlc.new_timestamp();
-        let future_time = now_ts.get_time() + NTP64::from(Duration::from_millis(500));
+        let future_time = now_ts.get_time() + NTP64::from(Duration::from_millis(1000));
         let future_ts = Timestamp::new(future_time, id.clone());
         assert!(hlc.update_with_timestamp(&future_ts).is_err())
     }
